#include "RgbeWriter.h"

#include <fstream>
using namespace std;

static void writeHeader(const char* file, int width, int height)
{
	ofstream out(file, ios::binary);
	char cr = 0x0a;
    out << "#?RGBE" << cr;
    out << "GAMMA=1.0" << cr;
    out << "EXPOSURE=1.0" << cr;
    out << "PRIMARIES=0.621 0.34 0.281 0.606 0.152 0.067 0.3127 0.329" << cr;
    out << "FORMAT=32-bit_rle_rgbe" << cr;
    out << cr;
    out << "-Y " << height << " +X " << width << cr;
    out.close();
}

static int floatToIntBits(float x) 
{
	float* floatPtr = &x;
	int* intPtr = (int*) floatPtr;
	return *intPtr;
}

static int exponent(float num)
{
	int value = floatToIntBits(num);
	int exp = ((value & (0x7f800000)) >> 23)-126;
	return exp;
}

static float normalize(float num)
{
	int value = floatToIntBits(num);
	int mantissa = value & (0x007fffff);
	float res = (float)(((double)(mantissa | 0x00800000))/((double)(0x01000000)));
	return res;
}

static void RGBToRGBE(float* rgb, int rgbByteOffset, char* rgbe, int rgbeOffset)
{
	float v;
	float n;
	int e;

	rgb = (float*)(((unsigned char*)rgb)+rgbByteOffset);

	v = rgb[0];
	if (rgb[1] > v) v = rgb[1];
	if (rgb[2] > v) v = rgb[2];
	
	if (v < 1e-32)
		rgbe[rgbeOffset+0] = rgbe[rgbeOffset+1] = rgbe[rgbeOffset+2] = rgbe[rgbeOffset+3] = 0;
	else
	{
		e = exponent(v);
		n = normalize(v);
		v = (float)(n * 256.0/v);
		
		// Clamp negative values to zero
		rgbe[rgbeOffset+0] = (char)((int)(rgb[0] * v) & 0xff);
		rgbe[rgbeOffset+1] = (char)((int)(rgb[1] * v) & 0xff);
		rgbe[rgbeOffset+2] = (char)((int)(rgb[2] * v) & 0xff);
		rgbe[rgbeOffset+3] = (char)((e + 128) & 0xff);
	}
}

// stride in bytes
static void writePixels(const char* file, int num, float* data, int stride)
{
	char* rgbeData = new char[num*4];
	int rgbByteOffset = 0;
	int rgbeOffset = 0;

	for (int i = 0; i < num; i++)
	{
		RGBToRGBE(data, rgbByteOffset, rgbeData, rgbeOffset);
		rgbByteOffset  += stride;
		rgbeOffset     += 4;
	}
    
	ofstream out(file, ios::binary | ios::app);
	out.write(rgbeData, num*4);
	out.close();
	delete rgbeData;
}

void RgbeWriter::writeFloatRgbBuffer(const char *file, int width, int height, float *data, int stride)
{
	writeHeader(file, width, height);
	writePixels(file, width * height, data, stride);
}

static void writeRgbePixels(const char* file, int num, unsigned int* data, int stride)
{
	char* rgbeData = new char[num*4];
	int srcByteOffset = 0;
	int rgbeOffset = 0;

	for (int i = 0; i < num; i++)
	{
		memcpy(rgbeData+rgbeOffset, ((unsigned char*)data)+srcByteOffset, 4);
		srcByteOffset  += stride;
		rgbeOffset += 4;
	}
    
	ofstream out(file, ios::binary | ios::app);
	out.write(rgbeData, num*4);
	out.close();
	delete rgbeData;
}

void RgbeWriter::writeRgbeBuffer(const char *file, int width, int height, unsigned int *data, int stride)
{
	writeHeader(file, width, height);
	writeRgbePixels(file, width * height, data, stride);
}